import { assignDeepOmitBlank, ChartMeasureRoleType, compact, getEntityProperty } from '@metad/ocap-core';
import { maxBy, minBy, range } from 'lodash-es';
import { getChromaticScale } from '../chromatics';
export function getColorLightnessVisualMap(measure, property, dataset, visualMaps) {
    const _dataset = Array.isArray(dataset) ? dataset[0] : dataset;
    const dataMin = minBy(_dataset.source, measure.measure)?.[measure.measure];
    const dataMax = maxBy(_dataset.source, measure.measure)?.[measure.measure];
    return {
        dimension: _dataset.dimensions ? _dataset.dimensions.indexOf(measure.measure) : measure.measure,
        min: dataMin,
        max: dataMax,
        text: [property.caption || property.name]
    };
}
/**
 * Calc visualMap component for dimension
 *
 * @param dimension
 * @param chartAnnotation
 */
export function dimensionVisualMaps(dimension, chartAnnotation, dataset, entityType) {
    const { measures } = chartAnnotation;
    const visualMaps = [];
    measures.forEach((chartMeasure) => {
        if (chartMeasure.palette) {
            visualMaps.push(getMeasurePaletteVisualMap(chartMeasure, dataset.source, getEntityProperty(entityType, chartMeasure)));
        }
        if (chartMeasure.role === ChartMeasureRoleType.Size) {
            visualMaps.push(getSymbolSizeVisualMap(chartMeasure, getEntityProperty(entityType, chartMeasure), dataset));
        }
    });
    return compact(visualMaps);
}
export function getMeasurePaletteVisualMap(chartMeasure, data, property) {
    if (!chartMeasure?.palette) {
        return null;
    }
    const { measure, palette, domain } = chartMeasure;
    const dataMin = minBy(data, measure)?.[measure];
    const dataMax = maxBy(data, measure)?.[measure];
    const dataSize = data.length;
    const _domain = domain || [dataMin, dataMax];
    const inRange = { color: [] };
    if (palette?.name) {
        const colorFun = getChromaticScale(palette.name, [0, dataSize - 1], palette.reverse);
        inRange.color = range(dataSize).map((i) => colorFun(i));
    }
    else if (palette?.color) {
        inRange.color = palette.color;
    }
    else {
        return null;
    }
    return assignDeepOmitBlank({
        show: false,
        type: 'continuous',
        text: [property.caption || property.name],
        min: _domain[0],
        max: _domain[1],
        inRange
    }, chartMeasure.chartOptions?.visualMap, Number.MAX_SAFE_INTEGER);
}
export function getSymbolSizeVisualMap(measure, property, dataset) {
    // 原来为什么注释掉, 是不是因为 dataset 情况变化
    const _dataset = Array.isArray(dataset) ? dataset[0] : dataset;
    const dataMin = minBy(_dataset.source, measure.measure)?.[measure.measure];
    const dataMax = maxBy(_dataset.source, measure.measure)?.[measure.measure];
    return assignDeepOmitBlank({
        dimension: _dataset.dimensions ? _dataset.dimensions.indexOf(measure.measure) : measure.measure,
        min: dataMin,
        max: dataMax,
        itemWidth: 30,
        itemHeight: 120,
        calculable: true,
        precision: 0.1,
        text: [property.caption || property.name],
        textGap: 30,
        inRange: {
            symbolSize: [10, 70]
        },
        outOfRange: {
            symbolSize: [10, 70],
            color: ['rgba(255,255,255,0.4)']
        },
        controller: {
            inRange: {
                color: ['#c23531']
            },
            outOfRange: {
                color: ['#999']
            }
        }
    }, measure.chartOptions?.visualMap, Number.MAX_SAFE_INTEGER);
}
//# sourceMappingURL=visualMap.js.map